# ------------------------------------------------------------------------
#   FUNCTIONS
# ------------------------------------------------------------------------

CustomDir () {
    $ECHO_n "
   I am about to create the \$HOME/.xcrysden directory and copy XCRYSDEN 
   definition/customization files to it. See the XCRYSDEN  documentation 
   for further instructions about XCRYSDEN customization.

   Press <Enter> to continue ... "

   read ans
   $CLEAR
}

ExeWarning () {
    if [ ! -x "$1" ]; then
	echo "WARNING: file $1 is not executable !!!"
    fi
}

QueryProgram() {
    # usage: QueryProgram msg1 mesg2 deafult variable
    $ECHO_n "$1 ([y]es/[n]o): "
    read ans; 
    if [ \( "$ans" != "y" \) -a \( "$ans" != "n" \) ]; then
	echo "Please answer y or n !!!"
	QueryProgram "$1" "$2" "$3" "$4"
	return
    elif [ "$ans" = "y" ]; then
	$ECHO_n "$2 (default: $3): "
	read prog;
	if [ "$prog" = "" ]; then
	    prog=$3
	fi
	ExeWarning "$prog"
	eval $4=\"$prog\"
    else
	unset $4
	#eval $4=""
    fi
}

GetYesNoAnswer() {
    # Usage: $0 text variable
    $ECHO_n "$1 ([y]es/[n]o): "
    read ans_

    if test \( "$ans_" != "y" \) -a \( "$ans_" != "n" \) ; then
	echo "Please answer y or n !!!"
	GetYesNoAnswer "$1" "$2"
	return
    else
	eval $2=\"$ans_\"
    fi
}

CustomDef () {
    #
    # Query the CRYSTAL program
    #
    echo "
CRYSTAL-95/98/03 is an electronic structure program for periodic systems.
(http://www.crystal.unito.it/)
"
    GetYesNoAnswer "Do you have a CRYSTAL-95/98/03 package" cXX

    echo "" > $tempdir/customFile.$$

    if [ "$cXX" = "y" ]; then	
	$ECHO_n "
Specify CRYSTAL integral module (e.g. \"crystal\" for CRYSTAL03)
(default: /usr/local/bin/crystal):  "
	read int
	if [ "$int" = "" ]; then
	    int=/usr/local/bin/crystal
	fi
	ExeWarning "$int"

	$ECHO_n "
Specify CRYSTAL scf module (e.g. \"crystal\" for CRYSTAL03)
(default: /usr/local/bin/crystal):  "
	read scf
	if [ "$scf" = "" ]; then
	    scf=/usr/local/bin/crystal
	fi
	ExeWarning "$scf"

	$ECHO_n "
Specify CRYSTAL properties module (default: /usr/local/bin/properties):  "
	read pro
	if [ "$pro" = "" ]; then
	    pro=/usr/local/bin/properties
	fi
	ExeWarning "$pro"
	echo "
# ------------------------------------------------------------------------
#  do we have CRYSTAL package
# ------------------------------------------------------------------------

set system(c95_exist) 1


# ------------------------------------------------------------------------
#  CRYSTAL modules
# ------------------------------------------------------------------------

set system(c95_integrals)  $int
set system(c95_scf)        $scf
set system(c95_properties) $pro

" >> $tempdir/customFile.$$
    else
	echo "
# ------------------------------------------------------------------------
#  do we have CRYSTAL package
# ---------------------------

set system(c95_exist) 0


# ------------------------------------------------------------------------
#  CRYSTAL modules
# ------------------------------------------------------------------------

#set system(c95_integrals)  /full/path/to/crystal
#set system(c95_scf)        /full/path/to/crystal
#set system(c95_properties) /full/path/to/properties

" >> $tempdir/customFile.$$
    fi

    # ------------------------------------------------------------------------
    #  Query PRINT command
    # ------------------------------------------------------------------------
    $ECHO_n "
Specify printing command (default: lpr -h):  "
    read lpr 
    if [ "$lpr" = "" ]; then
	lpr="lpr -h"
    fi

    
    #
    # babel, imagemagick(convert), whirlgif, mpeg_encode
    #
    echo "

XCRYSDEN uses a few external utility programs. These are the following:

   1.) the \"babel\" program is needed to display molecular structure from 
       Gaussian Z-matrix file.

   2.) the ImageMagick's \"convert\" program is needed to print-to-file in 
       JPG/PNG/GIF and other formats.

   3.) the \"whirlgif\" program is needed to create an Animated-GIF images 
       from the animation snapshots.

   4.) the \"mpeg_encode\" program is needed to create an MPEG movie 
       from the animation snapshots.

   Links
   --
   BABEL:       http://smog.com/chem/babel/
   IMAGEMAGICK: http://www.imagemagick.org/
   WHIRLGIF:    http://www.danbbs.dk/~dino/whirlgif/
   MPEG_ENCODE: http://bmrc.berkeley.edu/research/mpeg/mpeg_encode.html
" | $MORE

    # ------------------------------------------------------------------------
    #  Query BABEL program
    # ------------------------------------------------------------------------
    QueryProgram "Do you want to enable the \"babel\" program" \
	"Which BABEL program" "$BABEL" babel

    # ------------------------------------------------------------------------
    #  Query image conversion program
    # ------------------------------------------------------------------------
    QueryProgram "
Do you want to enable the ImageMagick's \"convert\" program" \
	"Which image conversion program" "$CONVERT" convert

    # ------------------------------------------------------------------------
    #  Query Whirlgif program
    # ------------------------------------------------------------------------
    QueryProgram "
Do you want to enable the \"whirlgif\" program" \
	"Which whirlgif program" "$WHIRLGIF" whirlgif

    # ------------------------------------------------------------------------
    #  Query mpeg_encode program
    # ------------------------------------------------------------------------
    QueryProgram "
Do you want to enable the \"mpeg_encode\" program" \
	"Which mpeg_encode program" "$MPEG_ENCODE" mpeg_encode 
        
    if [ "$babel" != "" ]; then
	_babel="set xcMisc(babel) $babel"
    else
	_babel="#set xcMisc(babel) /full/path/to/babel"
    fi

    if [ "$convert" != "" ]; then
	_convert="set xcMisc(ImageMagick.convert) $convert"
    else
	_convert="#set xcMisc(ImageMagick.convert) /full/path/to/convert"
    fi

    if [ "$whirlgif" != "" ]; then
	_whirlgif="set xcMisc(whirlgif) $whirlgif"
    else
	_whirlgif="#set xcMisc(whirlgif) /full/path/to/whirlgif"
    fi

    if [ "$mpeg_encode" != "" ]; then
	_mpeg_encode="set xcMisc(mpeg_encode) $mpeg_encode"
    else
	_mpeg_encode="#set xcMisc(mpeg_encode) /full/path/to/mpeg_encode"
    fi

    echo "

# ------------------------------------------------------------------------
#  print command
# ------------------------------------------------------------------------

set xcMisc(printCommand) {$lpr}


# ------------------------------------------------------------------------
#  An image conversion program: we need PPM to PNG/JPG/GIF/... conversion.
#  The \"convert\" program of ImageMagick (http://www.imagemagick.org/) is
#  a convinient choice.
# 
#  It is possible to specify the command-line options. For example:
#
#  set xcMisc(ImageMagick.convert) \"/usr/bin/convert \\
#                             -quality 90 -border 3x3 -bordercolor black\"
# 
# Instead one can also use something like:
#
#  set xcMisc(ImageMagick.convertOptions) \"-quality 90 -antialias \\
#                             -blur 1x1 -trim -bordercolor white \\
#                             -border 20x20 -bordercolor black -border 3x3\"
# ------------------------------------------------------------------------

$_convert 


# ------------------------------------------------------------------------
#  With \"whirlgif\" program XCRYSDEN can create an Animated-GIF image
# ------------------------------------------------------------------------

$_whirlgif


# ------------------------------------------------------------------------
#  MPEG_ENCODE for creating the MPEG movies
# ------------------------------------------------------------------------

$_mpeg_encode


# ------------------------------------------------------------------------
#  BABEL program
# ------------------------------------------------------------------------
$_babel
# this is the same as environmental BABEL_DIR variable. Note: if
# BABEL_DIR is defined, then this is not needed!!!  
#set xcMisc(babel_dir) \$env(HOME)/lib/babel


# ------------------------------------------------------------------------
# Custom setting of the atomic radii. The syntax is:
#
# set atmRad(atomic_number) radius
#
# Example:
#
# set atmRad(1) 0.5; # custom radius for Hydrogen
# set atmRad(8) 1.2; # custom radius for Oxygen
# ------------------------------------------------------------------------


# ------------------------------------------------------------------------
# Custom setting of the atomic colors. The syntax is:
#
# set atmCol(atomic_number) {red gren blue}
#
# The components (red,gren,blue) must be in range [0,1]
#
# Example:
#
# set atmCol(1) {0.5 0.5 0.5}; # custom color for Hydrogen
# set atmCol(8) {0.0 1.0 0.0}; # custom color for Oxygen
# ------------------------------------------------------------------------



# -------------------------------------------------------------------------
# Custom setting for a variety of molecular display parameters, such as
# ball-factors, specefill scale factors, tessellation factors, etc.
#
# Below are the default values. If you would like to change the
# default for a particular parameter, then uncomment the appropriate
# line and set the value according to your needs.
# ------------------------------------------------------------------------

## spacefill scale factor
#set myParam(ATRAD_SCALE)     1.40
#
## tesselation factor
#set myParam(TESSELLATION)    15.0 
#
## RGB color of unibonds (each compoenent must be within [0,1])
#set myParam(UNIBONDCOLOR)    {1.00 1.00 1.00} 
#
## Perspective Fovy, Front and Back parameters. The smaller the Fovy
## the larger the perception of perspective. Front and Back parameters
## determine the front and back clipping planes. The smaller the Back
## parameter the more the structure is clipped from the back side. The
## Front parameter is counter-intuitive, meaning the smaller it is the
## more the structure is clipped from the front side.
#
#set myParam(PERSPECTIVEFOVY)  2.5
#set myParam(PERSPECTIVEFRONT) 0.65
#set myParam(PERSPECTIVEBACK)  3.0
#
## ball-factor
#set myParam(BALLF)           0.4
#
## rod-factor
#set myParam(RODF)            0.6
#
## line-width of wireframe display-mode (in pixels)
#set myParam(WFLINEWIDTH)     1
#
## line-width of pointline display-mode (in pixels)
#set myParam(PLLINEWIDTH)     1
#
## line-width of crystal cell's frames
#set myParam(FRAMELINEWIDTH)  1
#
## Lighting-Off outline width
#set myParam(OUTLINEWIDTH)     1
#
## Lighting-On wire line width
#set myParam(WF3DLINEWIDTH)    1
#
## point-size of pointline display-mode (in pixels)
#set myParam(PLRADIUS)        6
#
## chemical connectivity factor
#set myParam(COV_SCALE)       1.05
#
## RGB color of crystal frame (each compoenent must be within [0,1])
#set myParam(FRAMECOL)        {0.88 1.00 0.67} 
#
## line-width of crystal frame
#set myParam(FRAMELINEWIDTH)  1 
#
## rod-factor of crystal frame
#set myParam(FRAMERODF)       0.1
#
## RGB background of XCRYSDEN display window 
## (each compoenent must be within [0,1])
#set myParam(BACKGROUND)      {0.00 0.00 0.00}
#
## maximum number of cells per direction for CRYSTALs
#set myParam(CRYSTAL_MAXCELL)  10
#
## maximum number of cells per direction for SLABs
#set myParam(SLAB_MAXCELL)     20
#
## maximum number of cells for POLYMERs
#set myParam(POLYMER_MAXCELL)  50
#
## default atomic-label's font (in X11 XLFD format)
#set myParam(ATOMIC_LABEL_FONT) -adobe-helvetica-medium-r-normal--12-120-75-75-p-67-iso8859-1
#
## default atomic-label's bright and dark color (in clamped-float RGB format)
#set myParam(ATOMIC_LABEL_BRIGHTCOLOR) {1.0 1.0 1.0}
#set myParam(ATOMIC_LABEL_DARKCOLOR)   {0.0 0.0 0.0}
#
#
## this are the parameters for the \"mpeg_encode\" program:
##--BEGIN::
#set myParam(MPEG_ENCODE_PARAM_FILE) {
#PATTERN          IBBPBBPBBPBBPBBP
#OUTPUT           \$output_file
#BASE_FILE_FORMAT PPM
#INPUT_FORMAT     UCB
#\$input_convert
#GOP_SIZE         16
#SLICES_PER_FRAME 1
#INPUT_DIR        \$input_dir
#INPUT
#\$input_files
#END_INPUT
#PIXEL           FULL
#RANGE           10
#PSEARCH_ALG     LOGARITHMIC
#BSEARCH_ALG     CROSS2
#IQSCALE         8
#PQSCALE         10
#BQSCALE         25
#REFERENCE_FRAME ORIGINAL
#BIT_RATE        1000000
#BUFFER_SIZE     327680
#FRAME_RATE      30
#}
##--END
#
# ------------------------------------------------------------------------



#
# ------------------------------------------------------------------------
#  Here go custom user-specified options
# ------------------------------------------------------------------------
#  In order to add an --unknown option to the Xcrysden allowed options,
#  do the following
#  
# Usage: 
#       addOption option converterProgram description
#
# Arguments:
#       option           ... option to add to XCRYSDEN options
#       converterProgram ... program that converts from an "unknown" to XSF format;
#                            this program must be supplied by the user !!!
#       description      ... description of the options that will appear in the
#                            help message (i.e. xcrysden --help).
# Example: 
#       addOption --unknown /home/tone/utils/unknown2xsf {
#               load structure from unknown file format
#       }
#
" >> $tempdir/customFile.$$

    echo "

 ========================================================================
           You have specified the following definitions:
 ========================================================================
"
    if [ "$cXX" = "y" ]; then
	echo "CRYSTAL integrals module:      $int
CRYSTAL scf module:            $scf
CRYSTAL properties module:     $pro"
    fi
    echo "Print Command:                 $lpr"
    if [ "$babel"       != "" ]; then echo "Babel program:                 $babel"; fi
    if [ "$convert"     != "" ]; then echo "Image converting program:      $convert"; fi
    if [ "$whirlgif"    != "" ]; then echo "Whirlgif program:              $whirlgif"; fi
    if [ "$mpeg_encode" != "" ]; then echo "Mpeg_encode program:           $mpeg_encode"; fi

    $ECHO_n "
Is this correct ([y]es/[n]o): "
    read cor
    if [ \( "$cor" != "y" \) -a \( "$cor" != "yes" \) ]; then
	$CLEAR
	echo "
You answered NO. 

Therefore, please answer again to all the questions !!!
"
	CustomDef
	return
    fi

    $ECHO_n "creating $HOME/.xcrysden/custom-definitions ... "
    cp $tempdir/customFile.$$ $HOME/.xcrysden/custom-definitions
    if [ $? -eq 0 ]; then
	echo OK
    else
	echo "
ERROR: failed to create $HOME/.xcrysden/custom-definitions file !!!"
    fi

    $ECHO_n "
Would you like to edit ~/.xcrysden/custom-definitions file now ([y]es/[n]o):"
    read edit
    if [ \( "$edit" = "y" \) -o \( "$edit" = "yes" \) ]; then
	${EDITOR:-vi} $HOME/.xcrysden/custom-definitions
    fi
}


# ------------------------------------------------------------------------
#   MAIN
# ------------------------------------------------------------------------

BABEL=`which babel 2> /dev/null`
CONVERT=`which convert 2> /dev/null`
WHIRLGIF=`which whirlgif 2> /dev/null`
MPEG_ENCODE=`which mpeg_encode 2> /dev/null`

BABEL=${BABEL:-/usr/bin/babel}
CONVERT=${CONVERT:-/usr/bin/convert}
WHIRLGIF=${WHIRLGIF:-/usr/bin/whirlgif}
MPEG_ENCODE=${MPEG_ENCODE:-/usr/bin/mpeg_encode}

if [ \( -d /tmp \) -a \( -w /tmp \) ]; then
    tempdir=/tmp
else
    # set tempdir to $HOME
    tempdir=$HOME
fi

CustomDir

echo "
   ------------------------------------------------------------------------
                 Please answer to following QUESTIONS!!!
   ------------------------------------------------------------------------
"

if [ ! -d $HOME/.xcrysden ]; then
    mkdir $HOME/.xcrysden
fi

CustomDef

$ECHO_n "copying $XCRYSDEN_TOPDIR/Tcl/Xcrysden_defaults ... "
cp $XCRYSDEN_TOPDIR/Tcl/Xcrysden_defaults $HOME/.xcrysden/Xcrysden_defaults
if [ $? -eq 0 ]; then
    echo OK
else
    echo "
ERROR: failed to copy $XCRYSDEN_TOPDIR/Xcrysden_defaults file !!!"
fi

if [ -f $tempdir/customFile.$$ ]; then
    rm -f $tempdir/customFile.$$
fi

echo "

FINAL NOTE: before running xcrysden please load (source) your profile, i.e.:

 - for sh:             . ~/.profile
 - for Bash:           . ~/.bashrc
 - for Csh or Tclsh:   source ~/.cshrc

   (( if you were instructed to manually edit your profile, please do so,
      and only then source the profile. ))

   Then type: xcrysden


   Please report BUGS to Tone.Kokalj@ijs.si. 

   TERMS OF USE:
   -------------
   XCRYSDEN is released under the GNU General Public License.
   
   Whenever graphics generated by XCRYSDEN are used in scientific
   publications, you should include at least the following reference:
   
   [ref] A. Kokalj: J. Mol. Graphics Modelling, 1999, Vol. 17,
         p. 176--179. Code available from http://www.xcrysden.org/.
   
   
   Good Luck !!! 
   
   XCRYSDEN tips: 
                  - try: xcrysden --help

                  - occasionally clean the xcrysden scratch directory
                    (you can use the \"xc_cleanscratch\" utility)
" | $MORE
